@@ -9,7 +9,6 @@ import Data.Int
 import Data.List as X
 import Data.Maybe
 import Data.Monoid
-import Data.Ord
 import System.IO
 
 import IO as System.IO
@@ -28,6 +27,13 @@ error = hPutChar stdout ==> putChar
 error = hPutStr stdout ==> putStr
 error = hPutStrLn stdout ==> putStrLn
 error = hPrint stdout ==> print
+error = hWaitForInput a 0 ==> hReady a
+error = hPutStrLn a (show b) ==> hPrint a b
+error = hIsEOF stdin ==> isEOF
+
+-- EXIT
+
+error = exitWith ExitSuccess ==> exitSuccess
 
 -- ORD
 
@@ -41,33 +47,33 @@ error = compare x y /= GT ==> x <= y
 error = compare x y == LT ==> x < y
 error = compare x y /= LT ==> x >= y
 error = compare x y == GT ==> x > y
-error = compare (f x) (f y) ==> Data.Ord.comparing f x y
-error = on compare f ==> Data.Ord.comparing f
-error = x == y || x == z ==> x `elem` [y,z]
-error = x /= y && x /= z ==> x `notElem` [y,z]
+error = x == a || x == b || x == c ==> x `elem` [a,b,c]
+error = x /= a && x /= b && x /= c ==> x `notElem` [a,b,c]
+--error = compare (f x) (f y) ==> Data.Ord.comparing f x y -- not that great
+--error = on compare f ==> Data.Ord.comparing f -- not that great
 
 -- READ/SHOW
 
 error = showsPrec 0 x "" ==> show x
 error = readsPrec 0 ==> reads
 error = showsPrec 0 ==> shows
+warn = showIntAtBase 16 intToDigit ==> showHex
+warn = showIntAtBase 8 intToDigit ==> showOct
 
 -- LIST
 
 error = concat (map f x) ==> concatMap f x
 warn = concat [a,b] ==> a ++ b
--- these hints pick up way too much
---warn = a++b++c++d++e++f++g ==> concat [a, b, c, d, e, f, g]
---warn = a++b++c++d++e++f ==> concat [a, b, c, d, e, f]
---warn = a++b++c++d++e ==> concat [a, b, c, d, e]
-error "Use map once" = map f (map g x) ==> map (f . g) x
+warn "Use map once" = map f (map g x) ==> map (f . g) x
 warn  = x !! 0 ==> head x
 error = take n (repeat x) ==> replicate n x
 error = head (reverse x) ==> last x
 error = head (drop n x) ==> x !! n
 error = reverse (tail (reverse x)) ==> init x
+error = take (length x - 1) x ==> init x
 error = isPrefixOf (reverse x) (reverse y) ==> isSuffixOf x y
 error = foldr (++) [] ==> concat
+error = foldl (++) [] ==> concat
 error = span (not . p) ==> break p
 error = break (not . p) ==> span p
 error = concatMap (++ "\n") ==> unlines
@@ -79,6 +85,7 @@ warn  = length x == 0 ==> null x where note = "increases laziness"
 warn  "Use null" = length x /= 0 ==> not (null x)
 error "Use :" = (\x -> [x]) ==> (:[])
 error = map (uncurry f) (zip x y) ==> zipWith f x y
+warn  = map f (zip x y) ==> zipWith (curry f) x y where _ = isVar f
 error = not (elem x y) ==> notElem x y
 warn  = foldr f z (map g x) ==> foldr (f . g) z x
 error = x ++ concatMap (' ':) y ==> unwords (x:y)
@@ -88,20 +95,42 @@ warn  = concat (intersperse " " x) ==> unwords x
 error "Use any" = null (filter f x) ==> not (any f x)
 error "Use any" = filter f x == [] ==> not (any f x)
 error = filter f x /= [] ==> any f x
+error = any ((==) a) ==> elem a
+error = any (== a) ==> elem a
+error = any (a ==) ==> elem a
+error = all ((/=) a) ==> notElem a
+error = all (/= a) ==> notElem a
+error = all (a /=) ==> notElem a
 
 -- FOLDS
 
-error = foldr (&&) True ==> and
-error = foldl (&&) True ==> and
-error = foldr (||) False ==> or
-error = foldl (||) False ==> or
-error = foldr (>>) (return ()) ==> sequence_
-error = foldl (+) 0 ==> sum
-error = foldl (*) 1 ==> product
+error = foldr  (>>) (return ()) ==> sequence_
+error = foldr  (&&) True ==> and
+error = foldl  (&&) True ==> and
+error = foldr1 (&&)  ==> and
+error = foldl1 (&&)  ==> and
+error = foldr  (||) False ==> or
+error = foldl  (||) False ==> or
+error = foldr1 (||)  ==> or
+error = foldl1 (||)  ==> or
+error = foldl  (+) 0 ==> sum
+error = foldr  (+) 0 ==> sum
+error = foldl1 (+)   ==> sum
+error = foldr1 (+)   ==> sum
+error = foldl  (*) 1 ==> product
+error = foldr  (*) 1 ==> product
+error = foldl1 (*)   ==> product
+error = foldr1 (*)   ==> product
+error = foldl1 max   ==> maximum
+error = foldr1 max   ==> maximum
+error = foldl1 min   ==> minimum
+error = foldr1 min   ==> minimum
+error = foldr mplus mzero ==> msum
 
 -- FUNCTION
 
 error = (\x -> x) ==> id
+error = (\x y -> x) ==> const
 error = (\(_,y) -> y) ==> snd
 error = (\(x,_) -> x) ==> fst
 warn "Use curry" = (\x y-> f (x,y)) ==> curry f where _ = notIn [x,y] f
@@ -110,13 +139,23 @@ error "Redundant $" = (($) . f) ==> f
 error "Redundant $" = (f $) ==> f
 warn  = (\x -> y) ==> const y where _ = isAtom y && notIn x y
 error "Redundant flip" = flip f x y ==> f y x where _ = isApp original
-error "Redundant id" = id x ==> x
-error "Redundant const" = const x y ==> x
+warn  = (\a b -> o (f a) (f b)) ==> o `Data.Function.on` f
+
+-- CHAR
+
+error = a >= 'a' && a <= 'z' ==> isAsciiLower a
+error = a >= 'A' && a <= 'Z' ==> isAsciiUpper a
+error = a >= '0' && a <= '9' ==> isDigit a
+error = a >= '0' && a <= '7' ==> isOctDigit a
+error = not (isControl a) ==> isPrint a
+error = isLower a || isUpper a ==> isAlpha a
+error = isAlpha a || isDigit a ==> isAlphaNum a
 
 -- BOOL
 
 error "Redundant ==" = a == True ==> a
 warn  "Redundant ==" = a == False ==> not a
+error "Redundant if" = (if a then x else x) ==> x where note = "reduces strictness"
 error "Redundant if" = (if a then True else False) ==> a
 error "Redundant if" = (if a then False else True) ==> not a
 error "Redundant if" = (if a then t else (if b then t else f)) ==> if a || b then t else f
@@ -127,6 +166,12 @@ warn  "Use if" = case a of {True -> t; False -> f} ==> if a then t else f
 warn  "Use if" = case a of {False -> f; True -> t} ==> if a then t else f
 warn  "Use if" = case a of {True -> t; _ -> f} ==> if a then t else f
 warn  "Use if" = case a of {False -> f; _ -> t} ==> if a then t else f
+warn  "Redundant if" = (if c then (True, x) else (False, x)) ==> (c, x) where note = "reduces strictness"
+warn  "Redundant if" = (if c then (False, x) else (True, x)) ==> (not c, x) where note = "reduces strictness"
+warn = or [x,y]  ==> x || y
+warn = or [x,y,z]  ==> x || y || z
+warn = and [x,y]  ==> x && y
+warn = and [x,y,z]  ==> x && y && z
 
 -- ARROW
 
@@ -137,6 +182,8 @@ warn  = (\(x,y) -> (f x, g y)) ==> f Control.Arrow.*** g where _ = notIn [x,y] [
 warn  = (\x -> (f x, g x)) ==> f Control.Arrow.&&& g where _ = notIn x [f,g]
 warn  = (\(x,y) -> (f x,y)) ==> Control.Arrow.first f where _ = notIn [x,y] f
 warn  = (\(x,y) -> (x,f y)) ==> Control.Arrow.second f where _ = notIn [x,y] f
+warn  = (f (fst x), g (snd x)) ==> (f Control.Arrow.*** g) x
+warn "Redundant pair" = (fst x, snd x) ==>  x
 
 -- FUNCTOR
 
@@ -147,7 +194,7 @@ error "Functor law" = fmap id ==> id
 
 error "Monad law, left identity" = return a >>= f ==> f a
 error "Monad law, right identity" = m >>= return ==> m
-warn  = m >>= return . f ==> fmap f m
+warn  = m >>= return . f ==> Control.Monad.liftM f m -- cannot be fmap, because is in Functor not Monad
 error = (if x then y else return ()) ==> Control.Monad.when x $ _noParen_ y where _ = not (isAtom y)
 error = (if x then y else return ()) ==> Control.Monad.when x y where _ = isAtom y
 error = (if x then return () else y) ==> Control.Monad.unless x $ _noParen_ y where _ = not (isAtom y)
@@ -156,9 +203,20 @@ error = sequence (map f x) ==> mapM f x
 error = sequence_ (map f x) ==> mapM_ f x
 warn  = flip mapM ==> Control.Monad.forM
 warn  = flip mapM_ ==> Control.Monad.forM_
+warn  = flip forM ==> mapM
+warn  = flip forM_ ==> mapM_
 error = when (not x) ==> unless x
 error = x >>= id ==> Control.Monad.join x
 error = liftM f (liftM g x) ==> liftM (f . g) x
+warn = a >> return () ==> void a
+warn = fmap (const ()) ==> void
+error = flip (>=>) ==> (<=<)
+error = flip (<=<) ==> (>=>)
+error = (\x -> f x >>= g) ==> f Control.Monad.>=> g where _ = notIn x [f,g]
+error = (\x -> f =<< g x) ==> f Control.Monad.<=< g where _ = notIn x [f,g]
+error = a >> forever a ==> forever a
+warn = liftM2 id ==> ap
+error = mapM (uncurry f) (zip l m) ==> zipWithM f l m
 
 -- MONAD LIST
 
@@ -170,6 +228,17 @@ error = sequence_ (replicate n x) ==> Control.Monad.replicateM_ n x
 error = mapM f (map g x) ==> mapM (f . g) x
 error = mapM_ f (map g x) ==> mapM_ (f . g) x
 
+-- APPLICATIVE / TRAVERSABLE
+
+error = flip traverse ==> for
+error = flip for ==> traverse
+error = flip traverse_ ==> for_
+error = flip for_ ==> traverse_
+error = foldr (*>) (pure ()) ==> sequenceA_
+error = foldr (<|>) empty ==> asum
+error = liftA2 (flip ($)) ==> (<**>)
+error = Just <$> a <|> pure Nothing ==> optional a
+
 -- LIST COMP
 
 warn  "Use list comprehension" = (if b then [x] else []) ==> [x | b]
@@ -179,6 +248,9 @@ warn  "Redundant list comprehension" = [x | x <- y] ==> y where _ = isVar x
 
 error "Redundant seq" = x `seq` x ==> x
 error "Redundant $!" = id $! x ==> x
+error "Redundant seq" = x `seq` y ==> y where _ = isWHNF x
+error "Redundant $!" = f $! x ==> f x where _ = isWHNF x
+error "Redundant evaluate" = evaluate x ==> return x where _ = isWHNF x
 
 -- MAYBE
 
@@ -189,15 +261,30 @@ error = not (isNothing x) ==> isJust x
 error = not (isJust x) ==> isNothing x
 error = maybe [] (:[]) ==> maybeToList
 error = catMaybes (map f x) ==> mapMaybe f x
+warn  = (case x of Nothing -> y; Just a -> a)  ==> fromMaybe y x
+error = (if isNothing x then y else f (fromJust x)) ==> maybe y f x
+error = (if isJust x then f (fromJust x) else y) ==> maybe y f x
+error = maybe Nothing (Just . f) ==> fmap f
+warn  = map fromJust . filter isJust  ==>  Data.Maybe.catMaybes
+error  = x == Nothing  ==>  isNothing x
+error  = Nothing == x  ==>  isNothing x
+error  = x /= Nothing  ==>  Data.Maybe.isJust x
+error  = Nothing /= x  ==>  Data.Maybe.isJust x
 error = concatMap (maybeToList . f) ==> Data.Maybe.mapMaybe f
 error = concatMap maybeToList ==> catMaybes
 error = maybe n Just x ==> Control.Monad.mplus x n
-warn  = (case x of Nothing -> y; Just a -> a)  ==> fromMaybe y x
 warn  = (case x of Just a -> a; Nothing -> y)  ==> fromMaybe y x
 error = (if isNothing x then y else fromJust x) ==> fromMaybe y x
 error = (if isJust x then fromJust x else y) ==> fromMaybe y x
 error = isJust x && (fromJust x == y) ==> x == Just y
 error = mapMaybe f (map g x) ==> mapMaybe (f . g) x
+error = fromMaybe a (fmap f x) ==> maybe a f x
+warn = [x | Just x <- a] ==> Data.Maybe.catMaybes a
+
+-- EITHER
+
+error = [a | Left a <- a] ==> lefts a
+error = [a | Right a <- a] ==> rights a
 
 -- INFIX
 
@@ -226,15 +313,32 @@ warn  = not (odd x) ==> even x
 warn  = x ** 0.5 ==> sqrt x
 warn  = x ^^ y ==> x ** y where _ = isLitInt y
 warn  "Use 1" = x ^ 0 ==> 1
+warn  = round (x - 0.5) ==> floor x
+
+-- CONCURRENT
+
+warn = mapM_ (writeChan a) ==> writeList2Chan a
 
 -- EXCEPTION
 
 error "Use Control.Exception.catch" = Prelude.catch ==> Control.Exception.catch where note = "Prelude.catch does not catch most exceptions"
 warn = flip Control.Exception.catch ==> handle
+warn = flip handle ==> Control.Exception.catch
 warn = flip (catchJust p) ==> handleJust p
+warn = flip (handleJust p) ==> catchJust p
 warn = Control.Exception.bracket b (const a) (const t) ==> Control.Exception.bracket_ b a t
 warn = Control.Exception.bracket (openFile x y) hClose ==> withFile x y
 warn = Control.Exception.bracket (openBinaryFile x y) hClose ==> withBinaryFile x y
+warn = throw (ErrorCall a) ==> error a
+error = a `seq` return a ==> Control.Exception.evaluate a
+error = toException NonTermination ==> nonTermination
+error = toException NestedAtomically ==> nestedAtomically
+
+
+-- WEAK POINTERS
+
+error = mkWeak a a b ==> mkWeakPtr a b
+error = mkWeak a (a, b) c ==> mkWeakPair a b c
 
 -- EVALUATE
 
@@ -273,6 +377,9 @@ error "Evaluate" = x * 1 ==> x
 error "Evaluate" = x / 1 ==> x
 error "Evaluate" = concat [a] ==> a
 error "Evaluate" = concat [] ==> []
+error "Evaluate" = zip [] [] ==> []
+error "Evaluate" = id x ==> x
+error "Evaluate" = const x y ==> x
 
 -- COMPLEX
 
@@ -304,7 +411,7 @@ yes = not (a == b) -- a /= b
 yes = not (a /= b) -- a == b
 yes = if a then 1 else if b then 1 else 2 -- if a || b then 1 else 2
 no  = if a then 1 else if b then 3 else 2
-yes = a >>= return . id -- fmap id a
+yes = a >>= return . id -- Control.Monad.liftM id a
 yes = (x !! 0) + (x !! 2) -- head x
 yes = if b < 42 then [a] else [] -- [a | b < 42]
 yes = take 5 (foo xs) == "hello" -- "hello" `Data.List.isPrefixOf` foo xs
@@ -363,6 +470,12 @@ yes = catch -- Control.Exception.catch
 import Prelude hiding (catch); no = catch
 import Control.Exception as E; no = E.catch
 main = do f; putStrLn $ show x -- print x
+main = map (writer,) $ map arcObj $ filter (rdfPredEq (Res dctreferences)) ts -- map ((writer,) . arcObj) (filter (rdfPredEq (Res dctreferences)) ts)
+h x y = return $! (x, y) -- return (x, y)
+h x y = return $! x
+getInt = do { x <- readIO "0"; return $! (x :: Int) }
+foo = evaluate [12] -- return [12]
+test = \ a -> f a >>= \ b -> return (a, b)
 
 import Prelude \
 yes = flip mapM -- Control.Monad.forM
